using UnityEngine;
using System.Collections;
using UnityEngine.Video;
using UnityEngine.UI;
using System.Linq;
using System;
using System.Text;
using System.Diagnostics;
using UnityEngine.EventSystems;
using System.Collections.Generic;

namespace qdaro {
class UITweaks : MVRScript {
	private List<RaycastResult> castList = new List<RaycastResult>();
	private List<string> tabbableContainerNames = new List<string>{"Male Morphs", "Female Morphs", "Clothing", "Hair", "Skin Select"};
	private List<string> morphContainerNames = new List<string>{"Male Morphs", "Female Morphs"};
	private List<string> tabbedFinderTerminants = new List<string>{"Scroll View", "MorphAutoUIMaleTabButtons"};
	private Dictionary<string, float> morphValues = new Dictionary<string, float>();
	private JSONStorableBool wheelTabNavJSON;
	private JSONStorableFloat tabbingShiftMultiplierJSON;
	private JSONStorableBool secondaryResetsSlidersJSON;
	private JSONStorableBool middleTogglesMorphsJSON;

	public override void Init() {
		JSONStorableString infoJSON = new JSONStorableString("Info",
			"Use mouse wheel to navigate between tabs on any tabbable area, such as clothing, hair, morphs, etc.\n\n"
			+ "Hold Shift to navigate by shift modifier.\n\n"
			+ "Right click to reset sliders or trigger morphs also works when clicking anywhere on the whole morph box, you don't have to aim specifically at the slider.\n\n"
		);
		UIDynamic textfield = this.CreateTextField(infoJSON, true);
		textfield.height = 600;

		wheelTabNavJSON = new JSONStorableBool("Wheel Tab Navigation", true);
		wheelTabNavJSON.storeType = JSONStorableParam.StoreType.Full;
		this.CreateToggle(wheelTabNavJSON, false);
		this.RegisterBool(wheelTabNavJSON);

		tabbingShiftMultiplierJSON = new JSONStorableFloat("Wheel Nav Shift Multiplier", 10, 1, 20, true, true);
		tabbingShiftMultiplierJSON.storeType = JSONStorableParam.StoreType.Full;
		UIDynamicSlider tabbingShiftSlider = this.CreateSlider(tabbingShiftMultiplierJSON, false);
		tabbingShiftSlider.slider.wholeNumbers = true;
		tabbingShiftSlider.valueFormat = "F0";
		this.RegisterFloat(tabbingShiftMultiplierJSON);

		secondaryResetsSlidersJSON = new JSONStorableBool("Secondary click resets sliders", true);
		secondaryResetsSlidersJSON.storeType = JSONStorableParam.StoreType.Full;
		this.CreateToggle(secondaryResetsSlidersJSON, false);
		this.RegisterBool(secondaryResetsSlidersJSON);

		middleTogglesMorphsJSON = new JSONStorableBool("Middle click toggles morphs", true);
		middleTogglesMorphsJSON.storeType = JSONStorableParam.StoreType.Full;
		this.CreateToggle(middleTogglesMorphsJSON, false);
		this.RegisterBool(middleTogglesMorphsJSON);
	}

	private void Update() {
		if (EventSystem.current.IsPointerOverGameObject()) {
			// Wheel tab navigation
			if (wheelTabNavJSON.val) {
				float x = Input.GetAxis("Mouse ScrollWheel");
				if (x != 0) {
					GameObject container = GetFirstHoveredObjectByNames(tabbableContainerNames, tabbedFinderTerminants);
					if (container != null) {
						GenerateTabbedUI tabbed = container.GetComponentInChildren<GenerateTabbedUI>();
						if (tabbed != null) {
							int delta = (Input.GetKey(KeyCode.LeftShift) || Input.GetKey(KeyCode.RightShift))
								? (int) tabbingShiftMultiplierJSON.val : 1;
							TabbedNavBy(tabbed, x > 0 ? -delta : delta);
						}
					}
					return;
				}
			}

			// Reset sliders on secondary mouse click
			if (secondaryResetsSlidersJSON.val && Input.GetKeyDown(KeyCode.Mouse1)) {
				Slider slider = GetFirstHoveredComponent<Slider>();

				// Retrieve slider from morph panels
				if (slider == null) {
					var morph = GetMorphComponentUnderCursor();
					if (morph != null) slider = morph.GetComponentInChildren<Slider>();
				}

				if (slider != null) slider.value = 0;
				return;
			}

			// Toggle morphs on middle click
			if (middleTogglesMorphsJSON.val && Input.GetKeyDown(KeyCode.Mouse2)) {
				var morph = GetMorphComponentUnderCursor();
				if (morph != null) {
					Slider slider = morph.GetComponentInChildren<Slider>();
					var texts = morph.GetComponentsInChildren<Text>();

					if (slider != null && texts.Length > 1) {
						// We construct a morph faux-ID from first two texts in its container component.
						// These usually hold var version + morph name, or morph name + category on local morphs.
						string fid = texts[0].text + texts[1].text;

						// Recall last value if any
						if (slider.value == 0) {
							if (morphValues.ContainsKey(fid)) slider.value = morphValues[fid];
						}
						// Remember current, and set to 0
						else {
							morphValues[fid] = slider.value;
							slider.value = 0;
						}
					}
				}
				return;
			}
		}
	}

	private Component GetMorphComponentUnderCursor() {
		var panel = GetFirstHoveredObjectByName("Panel");
		var parent = panel.transform.parent;
		if (parent != null && parent.name.StartsWith("Morph")) {
			return parent;
		}
		return null;
	}

	private List<RaycastResult> GetObjectsUnderCursor() {
		castList.Clear();
		PointerEventData eventData = new PointerEventData(EventSystem.current);
		eventData.pressPosition = Input.mousePosition;
		eventData.position = Input.mousePosition;
		EventSystem.current.RaycastAll(eventData, castList);
		return castList;
	}

	private GameObject GetFirstHoveredObjectByName(string nameFilter, List<string> stopAt = null) {
		List<RaycastResult> castList = GetObjectsUnderCursor();
		if (castList.Count > 0) {
			for (int i = 0; i < castList.Count; i++) {
				if (stopAt != null && stopAt.Contains(castList[i].gameObject.name)) {
					return null;
				}
				if (castList[i].gameObject.name == nameFilter) {
					return castList[i].gameObject;
				}
			}
		}
		return null;
	}

	private GameObject GetFirstHoveredObjectByNames(List<string> names, List<string> stopAt = null) {
		List<RaycastResult> castList = GetObjectsUnderCursor();
		if (castList.Count > 0) {
			for (int i = 0; i < castList.Count; i++) {
				if (stopAt != null && stopAt.Contains(castList[i].gameObject.name)) {
					return null;
				}
				if (names.Contains(castList[i].gameObject.name)) {
					return castList[i].gameObject;
				}
			}
		}
		return null;
	}

	private T GetFirstHoveredComponent<T>(string objectNameFilter = null) where T : UnityEngine.Component {
		List<RaycastResult> castList = GetObjectsUnderCursor();
		if (castList.Count > 0) {
			for (int i = 0; i < castList.Count; i++) {
				T component = castList[i].gameObject.GetComponent<T>();
				if (component != null) {
					return component;
				}
			}
		}
		return null;
	}

	private void TabbedNavBy(GenerateTabbedUI tabbed, int delta) {
		int tabCount = tabbed.tabButtonParent.childCount;

		if (tabCount == 0) return;

		// Get current tab index
		int tabIndex = 0;
		for (int i = 0; i < tabCount; i++) {
			Toggle t = tabbed.tabButtonParent.GetChild(i).GetComponentInChildren<Toggle>();
			if (t.isOn == true) {
				tabIndex = i + 1;
				break;
			}
		}

		TabbedNavTo(tabbed, Math.Max(1, Math.Min(tabIndex + delta, tabCount)));
	}

	private void TabbedNavTo(GenerateTabbedUI tabbed, int tabIndex) {
		int tabCount = tabbed.tabButtonParent.childCount;

		if (tabIndex < 1 || tabIndex > tabCount) return;

		tabbed.GotoTab(tabIndex);

		// Scroll to new active tab
		ScrollRect sl = tabbed.tabButtonParent.GetComponentInParent<ScrollRect>();
		if (sl != null) {
			sl.horizontalScrollbar.value = (float)Math.Max(0, tabIndex - 1) / Math.Max(1, tabbed.tabButtonParent.childCount - 1);
		}
	}

	private void OnDestroy() {
		// Placeholder
	}
}
}